//
// Created by spenc on 5/19/2025.
//

#include "LuaCpp.h"
#include <iostream>
#include <functional>
#include <vector>
#include <regex>

#include <cmath>
#include <fstream>
#include <sstream>
#include <string>

#include <cstdio>

#ifndef NDEBUG
#define LC_LOG(fmt, ...) std::printf("[Lua] " fmt "\n", ##__VA_ARGS__)
#else
    #define LC_LOG(fmt, ...) ((void)0)
#endif

std::ostream& operator<<(std::ostream& os, const std::unordered_map<std::string, uint8_t>& map) {
    os << "{\n";
    for (const auto& [key, value] : map) {
        os << "  \"" << key << "\": " << static_cast<int>(value) << ",\n";
    }
    os << "}";
    return os;
}


namespace LC
{
    // Public API
    LuaStatus LuaCpp::Call(const std::string &functionName)
    {

        if (!luaFunctions.contains(functionName)) {
            LC_LOG("Not Found '%s'", functionName.c_str());
            return LC_FUNCTIONNOTFOUND;
        }




        LC_LOG("Called %s", functionName.c_str());
        return LC_OK;
    }

    LuaStatus LuaCpp::loadFile(const std::string &scriptPath)
    {
        luaFunctions.clear();

        path = scriptPath;
        LuaStatus status = LoadInternal();

        if (!status.Ok()) {
            printf("[LuaCpp] %s", status.ToString().c_str());
            return status;
        }

        initialized = true;
        return LC_OK;
    }


    // Private API
    LuaCpp::LuaCpp()
    {
        luaFunctions.clear();
    }

    void LuaCpp::ExtractLuaFunctions(const std::string &luaCode)
    {
        std::regex funcRegex(R"(\bfunction\s+([a-zA-Z_][\w.:]*)\b)");
        std::smatch match;
        auto begin = luaCode.cbegin();
        auto end = luaCode.cend();

        while (std::regex_search(begin, end, match, funcRegex)) {
            std::string functionName = match[1];
            luaFunctions[functionName] = 0; // placeholder value
            begin = match.suffix().first;
        }
    }

    void LuaCpp::ExtractLuaVariables(const std::string& luaCode)
    {
        std::regex varRegex(R"(\b([a-zA-Z_][\w]*)\s*=\s*[^=])"); // avoids '=='
        std::smatch match;
        auto begin = luaCode.cbegin();
        auto end = luaCode.cend();

        while (std::regex_search(begin, end, match, varRegex)) {
            std::string varName = match[1];
            luaVariables[varName] = 0;  // Placeholder bytecode pointer
            begin = match.suffix().first;
        }


    }



    std::string LuaCpp::ReadFile(const std::string &path)
    {
        const std::ifstream file(path, std::ios::in | std::ios::binary);
        if (!file) {
            return "";
        }
        std::ostringstream contents;
        contents << file.rdbuf();
        return contents.str();
    }

    LuaStatus LuaCpp::LoadInternal()
    {
        std::string fileContents = ReadFile(path);


        if (fileContents.empty()) {
            return LC_FILEREADERROR;
        }
        ExtractLuaVariables(fileContents);
        ExtractLuaFunctions(fileContents);

        std::cout << luaFunctions << std::endl;
        std::cout << luaVariables << std::endl;

        return LC_OK;
    }
} // LC
