//
// Created by spenc on 5/19/2025.
//

#ifndef LUACPP_H
#define LUACPP_H
#include <unordered_map>
#include <string>
#include <cstdint>

namespace LC
{
    enum LuaStatusCode {
        LC_OK = 0,
        LC_ERROR,
        LC_UNINITIALIZED,
        LC_UNAVAILABLE,
        LC_UNAUTHORIZED,
        LC_UNSUPPORTED,
        LC_NOFILE,
        LC_FILEREADERROR,
        LC_FUNCTIONNOTFOUND,
    };

    class LuaStatus {
    public:
        LuaStatus(LuaStatusCode code = LC_OK) : code_(code) {}

        [[nodiscard]] bool Ok() const { return code_ == LC_OK; }

        [[nodiscard]] LuaStatusCode Code() const { return code_; }

        [[nodiscard]] std::string ToString() const {
            switch (code_) {
                case LC_OK: return "OK";
                case LC_ERROR: return "Error";
                case LC_UNINITIALIZED: return "Uninitialized";
                case LC_UNAVAILABLE: return "Unavailable";
                case LC_UNAUTHORIZED: return "Unauthorized";
                case LC_UNSUPPORTED: return "Unsupported";
                case LC_NOFILE: return "No File";
                case LC_FILEREADERROR: return "File Read Error";
                case LC_FUNCTIONNOTFOUND: return "Function Not Found";

                default: return "Unknown";
            }
        }

        bool operator==(LuaStatusCode other) const { return code_ == other; }
        bool operator!=(LuaStatusCode other) const { return code_ != other; }

    private:
        LuaStatusCode code_;
    };



    class LuaCpp
    {
    public:

        LuaCpp();
        ~LuaCpp() = default;
        LuaCpp(const LuaCpp&) = delete;


        LuaStatus loadFile(const std::string& scriptPath);
        LuaStatus Call(const std::string& functionName);


    private:
        // Private Functions
        void ExtractLuaFunctions(const std::string& luaCode);
        void ExtractLuaVariables(const std::string& luaCode);
        LuaStatus LoadInternal();
        static std::string ReadFile(const std::string& path);



        // Private Vars (alphabetical by and by type)
        bool initialized = false;

        std::unordered_map<std::string, uint8_t> luaFunctions;
        std::unordered_map<std::string, uint8_t> luaVariables;


        std::string path;
    };
} // LC

#endif //LUACPP_H
